% romsos -- Reduced-order model retaining second order form
%
% For the SISO second order system with input u and output y,
%   Ma + Dv + Kx = Pu
%   y = Vx
% romsos generates matrices Mk, Dk, Kk, Pk, and Vk by projection
% onto a subspace generated by a block-Lanczos type process.
% 
% This routine is used by the user routine cho_ss_romsos.
%
% Inputs:
%   n  -- original state space dimension
%   nb -- blocking factor
%   k  -- size of Krylov subspace (size of reduced model)
%   M,D,K,P,V -- original system matrices
%
% Outputs:
%   Mk,Dk,Kk,Pk,Vk -- system matrices for reduced model
%   L -- matrix mapping states in the reduced model to states in the original

function [Mk,Dk,Kk,Pk,Vk,L]=romsos(n,nb,k,M,D,K,P,V); 
%
% currently, this version only admits the expansion point at s0 = 0; 
% in addition, matrix K should be symmetric positive definite. 
% 
[Qd,Qv]=dkmr(n,nb,k,M,D,K,P,V); 
L = Qd; 
Mk = L'*M*L;  
Dk = L'*D*L;  
Kk = L'*K*L;  
Pk = L'*P; 
Vk = L'*V;  
