%Sugar tutorial of static analysis of a single beam.
%by JVClark, Oct2000

%intro page
clc
fprintf(['\n\n'...
' SUGAR demo - Static Analysis of a Cantilever Beam\n\n\n'...
' This demo consists of:\n'...
'   1) Netlist of a cantilever beam subjected to an external force\n'...
'   2) Static analysis of deflection\n'...
'   3) Display of structure\n'...
'   4) Displacement values\n'...
'\n\n\n\n\n Press key to continue.  ']); 
pause;

%1) netlist page
clc
fprintf('\n\n 1/4) Netlist file: cantilever.net\n\n');
type cantilever.net
fprintf(['\n\n'...
' Netlist description\n\n'...
' In the above netlist, the first element is the anchor model.\n'...
'    The anchor uses the poly1 (p1) layer information from the process file.\n'...
'    This anchor model has one node called substrate.\n'...
'    Geometrical parameters define length(l) width(w), and rotation about the z-axis (oz).\n\n'...
' The second element is the beam model called beam3d.\n'...
'    This model consists of 2 nodes, which go from substrate to tip.\n'...
'    This beam and the above anchor are coupled at the shared node (substrate).\n'...
'    The geometrical parameters follow from the anchor element.\n\n'...
' The final element is an external force.\n'...
'    The force is applied on the beam''s end node (tip).\n'...
'    The parameters define the force value (F) and orientation about the z-axis (oz).\n'...
'\n\n\n\n\n Press key to continue.  ']); 
pause

%2) static analysis page
clc
fprintf(['\n\n'...
' 2/4) Static analysis for deflection.\n\n'...
' These Matlab commands will load the netlist, then calculate its deflection:\n\n\n']);
echo on;
net = cho_load('cantilever3.net');
dq = cho_dc(net);
echo off;
fprintf('\n\n\n\n\n Press key to continue.  '); 
pause

%3) display page
clc
fprintf(['\n\n'...
' 3/4) Display of structure.\n\n'...
' Use these commands to display the structure before and after applied forces:\n\n\n']);
echo on;
figure(1); cho_display(net);
figure(2); cho_display(net,dq);
echo off;
fprintf('\n\n\n\n\n Press key to continue.  '); 
pause
clc;

%4) Displacement value page
clc
fprintf(['\n\n'...
' 4/4) Displacement value.\n\n'...
' The following command will give the displacement value of a particular node & coordinate (x,y,z,rx,ry,rz):\n\n\n'...
' dy = dqval(net,dq,''tip'',''y'')\n\n'...
' where tip is the node name, and y is desired coordinate\n\n'...
' Note: Though netlist angles are degrees, output displacement angles of this function are radians. xyz displacements are meters.\n\n'...
'\n\n\n\n\n Press key to continue.  \n']); 
pause
echo on;
dy = dqval(net,dq,'tip','y')
echo off;
fprintf('\nEnd of demo\n');

